; ### Hat Trick ###
; Port of the C64 game Hat Trick for VIC-20.
; For 16k.
; Uses 24k for player 2 gfx.

; $0200 - $02bf : player 1 spr addr h LUT
; $02c0 - $02e0 : y to char LUT
; $0304 - $030b : player graphics y offset LUT
; $030c - $0313 : player graphics height LUT
; $0340 - $03ff : player 1 spr addr l LUT
; $1000 - $11ff : vmatrix, phase 1 init code
; $1201 - $xxxx : basic start, ml entry, code/data (low)
; $1c00 - $1xxx : charset (static)
; $1xxx - $1yyy : charset (dynamic)
; $1xxx - $1zzz : phase 2 init code
; $1zzz - $3xxx : code/data (high)
; $3xxx - $3yyy : speedcode ($403 bytes)
; $3xxx - $3zzz : sprite source data ($3f0 bytes)
; $3zzz - $5xxx : phase 0 init code, phase 1 code source
; $3yyy - $54b7 : player 1 graphics
; $5c00 - $7xxx : player 2 graphics
; $7e40 - $7fff : player 2 spr addr LUTs

!ct "text_convtab.bin"

; # const

DEBUG_IRQ_LEN = 0
DEBUG_NMI_LEN = 0

OP_STA_ABS = $8d
OP_RTS = $60
OP_BIT_ABS = $2c

SOUND_VOLUME = 8

COLS = 21
ROWS = 21

AREA_W = 21
AREA_H = 20
AREA_YOFF = 1

AREA_GOAL_B = 8
AREA_GOAL_E = 12

CHAR_END = CHAR_BASE_GFX + 4 + 3 * 4 * 2 + 3 * 2

Sprite_h_player = 21
Sprite_h_goalie = 12
Sprite_h_puck = 4

Sprite_num = 8 * 8 * 3  ; xshift * dir * frame

CO_VIC_OFFS_X = 2
CO_VIC_OFFS_Y = -24
CO_VIC_OFFS_MAX_X = 4

CO_CPU_SHOOT_MIN_X = 45 + CO_VIC_OFFS_X
CO_CPU_SHOOT_MAX_X = 113 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_CPU_SHOOT_MIN_Y = CO_GOA_MIN_Y
CO_CPU_SHOOT_MAX_Y = CO_GOA_MAX_Y
CO_GOA_MIN_Y = 86 + CO_VIC_OFFS_Y
CO_GOA_MAX_Y = 113 + CO_VIC_OFFS_Y
CO_PL1_START_X = 50 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X / 2
CO_PL2_START_X = 100 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X / 2
CO_PLG_START_Y = 100 + CO_VIC_OFFS_Y
CO_PLN_MIN_X = 7 + CO_VIC_OFFS_X
CO_PLN_MAX_X = 141 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_PLN_MIN_Y = 40 + CO_VIC_OFFS_Y
CO_PLN_MAX_Y = 163 + CO_VIC_OFFS_Y
CO_PLP_MIN_X = 10 + CO_VIC_OFFS_X
CO_PLP_MAX_X = 138 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_PLP_MIN_Y = 44 + CO_VIC_OFFS_Y
CO_PLP_MAX_Y = 159 + CO_VIC_OFFS_Y
CO_PL_DIR_TH1_Y = 77 + CO_VIC_OFFS_Y
CO_PL_DIR_TH2_Y = 125 + CO_VIC_OFFS_Y
CO_PL_DIR_TH_X = 74 + CO_VIC_OFFS_X
CO_PUCKG_MIN_X = 2 + CO_VIC_OFFS_X
CO_PUCKG_MAX_X = 156 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_PUCKN_MIN_X = 6 + CO_VIC_OFFS_X
CO_PUCKN_MAX_X = 150 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_PUCK_IN_G_MIN_Y = 91 + CO_VIC_OFFS_Y
CO_PUCK_IN_G_MAX_Y = 129 + CO_VIC_OFFS_Y
CO_PUCK_L_MIN_X = 7 + CO_VIC_OFFS_X
CO_PUCK_L_MAX_X = 13 + CO_VIC_OFFS_X
CO_PUCK_MIN_Y = 40 + CO_VIC_OFFS_Y
CO_PUCK_MAX_Y = 180 + CO_VIC_OFFS_Y
CO_PUCK_R_MIN_X = 146 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_PUCK_R_MAX_X = 151 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X
CO_PUCK_START_X = 79 + CO_VIC_OFFS_X + CO_VIC_OFFS_MAX_X / 2
CO_PUCK_START_Y_0 = 39 + CO_VIC_OFFS_Y
CO_PUCK_START_Y_1 = 181 + CO_VIC_OFFS_Y

; # addr

zpsrc = $02
zpsrch = $03
zpdst = $04
zpdsth = $05

; used by init phase 0
Temp_lines = $06
temp_spri = $06
temp_ndir = $07
temp_nframe = $08
temp_nxshift = $09
temp_tptr = $0a
temp_tptrh = $0b
temp_sprh = $0c
temp_spryoff = $0d
temp_sprylast = $0e
temp_gfx_c0 = $10
temp_gfx_c1 = temp_gfx_c0 + Sprite_h_player
temp_gfx_c2 = temp_gfx_c1 + Sprite_h_player

; used by drawing code
temp_x = $10
temp_xs = $11
temp_f = $12
temp_cw = $13   ; x width in chars
temp_ys = $14
temp_ch = $15
temp_cl = $17

flag_wait_for_irq = $18
flag_pl2_userport_joy = flag_wait_for_irq + 1
flag_read_pl2 = flag_pl2_userport_joy + 1
temp_y = flag_read_pl2 + 1
temp_foo3_minus_x = temp_y + 1
temp_foo3_minus_y = temp_foo3_minus_x + 1
temp_hmm = temp_foo3_minus_y + 1
l0415 = temp_hmm + 1
l0416 = l0415 + 1
temp_num_bcd = l0416 + 12
temp_num_width = temp_num_bcd + 1
temp_char_width = temp_num_width + 1
temp_char_line_counter = temp_char_width + 1
p1_acc_x = temp_char_line_counter + 3
p1_vel_x = p1_acc_x + 1
p2_acc_x = p1_vel_x + 1
p2_vel_x = p2_acc_x + 1
p1_acc_y = p2_vel_x + 1
p1_vel_y = p1_acc_y + 1
p2_acc_y = p1_vel_y + 1
p2_vel_y = p2_acc_y + 1
p1_apply_counter = p2_vel_y + 1
p2_apply_counter = p1_apply_counter + 1
p1_coord_x = p2_apply_counter + 1
p2_coord_x = p1_coord_x + 1
p1_coord_y = p2_coord_x + 1
p2_coord_y = p1_coord_y + 1
p1_coord_y_cooked = p2_coord_y + 1
p2_coord_y_cooked = p1_coord_y_cooked + 1
p1_spr_h = p2_coord_y_cooked + 1
p2_spr_h = p1_spr_h + 1
px_spr_h = p2_spr_h + 1
p1_dir_hmm = px_spr_h  + 1
p1_dir_i = p1_dir_hmm + 1
p2_dir_hmm = p1_dir_i + 1
p2_dir_i = p2_dir_hmm + 1
g1_coord_y = p2_dir_i + 1
g2_coord_y = g1_coord_y + 1
puck_coord_x = g2_coord_y + 1
puck_coord_y = puck_coord_x + 1
puck_acc_x = puck_coord_y + 1
puck_vel_x = puck_acc_x + 1
puck_acc_y = puck_vel_x + 1
puck_vel_y = puck_acc_y + 1
phase_hmm = puck_vel_y + 1
l044a = phase_hmm + 1
l044b = l044a + 1
pX_has_puck = l044b + 1
puck_holder = pX_has_puck + 1
counter_hmm_sub = puck_holder + 1
counter_hmm = counter_hmm_sub + 1
saves_p1 = counter_hmm + 1
saves_p2 = saves_p1 + 1
flag_overtime = saves_p2 + 1
flag_music = flag_overtime + 1
flag_num_players = flag_music + 1
flag_skill = flag_num_players + 1
flag_cpu_player = flag_skill + 1
p1_anim_subframe = flag_cpu_player + 1
p1_anim_frame = p1_anim_subframe + 1
p1_anim_cooked = p1_anim_frame + 1
p2_anim_subframe = p1_anim_cooked + 1
p2_anim_frame = p2_anim_subframe + 1
p2_anim_cooked = p2_anim_frame + 1
puck_free_counter = p2_anim_cooked + 1
timeout_check_collider_puck_p1 = puck_free_counter + 1
timeout_check_collider_puck_p2 = timeout_check_collider_puck_p1 + 1
timeout_check_coll_p1_p2 = timeout_check_collider_puck_p2 + 1
score_bcd_p1 = timeout_check_coll_p1_p2 + 1
score_bcd_p2 = score_bcd_p1 + 1
timer_m = score_bcd_p2 + 1
timer_s = timer_m + 1
timer_sub_60 = timer_s + 1
flag_skip_irq_logic = timer_sub_60 + 1
flag_skip_irq_logic2 = flag_skip_irq_logic + 1
Music_zp_base = flag_skip_irq_logic2 + 1

temp_spri_tbl = $100

; address of player 1 sprites
; index is (frame << 6) | (direction << 3) | xshift
; table h must be at $xx00
tbl_spraddr_h = $0200
tbl_spraddr_l = $0340

tbl_y_char_l = $02c0
tbl_y_char_h = tbl_y_char_l + ROWS

; player sprite y offset for each direction
tbl_spryoff = $0304
; player sprite height for each direction
tbl_sprh = $030c

vmatrix = $1000
charset = $1c00
Charset_end = charset + CHAR_END * 8
charset_rom = $8000
colorram = $9400 + (vmatrix & $200)

Speedcode_len = (AREA_W - 2) * (AREA_H - 2) * 3 + 1
Speedcode_end = draw_clear + Speedcode_len

; player sprites
; 3 frames, 8 directions, 8 xshifts, <=21 lines tall
; xshift 0..3 are 2 bytes wide
; xshift 4..7 are 3 bytes wide

Spritegfx_max_len = 3 * 8 * (4 * 2 + 4 * 3) * Sprite_h_player

Spritegfx = Speedcode_end

RAM_16k_end = $6000

; extra data if 24k is available

; HACK the start address should depend on player graphics lengts, which depends on the data itself
Data_ext2_start = $5a00
Spritegfx2 = Data_ext2_start
; table h must be at $xx00
tbl_spraddr2_h = $7f00
tbl_spraddr2_l = $7e40

RAM_24k_end = $8000


; ###############

; # code entry

* = 4621

ml_entry:
    jmp init_phase0


; # irq handlers

hook_irq_handler_ingame:
    lda #0
    sta flag_irq_handler_ingame_skip
    jmp wait_for_irq_x2

irq_handler_ingame:
    lda $9124   ; ACK IRQ
; DEBUG
!if DEBUG_IRQ_LEN > 0 {
    lda #$1a
    sta $900f
}
    lda #$00
    sta flag_wait_for_irq
flag_irq_handler_ingame_skip = * + 1
    lda #1
    bne irq_handler_ingame_exit
    lda flag_skip_irq_logic
    bne l0570
    lda flag_skip_irq_logic2
    bne irq_handler_ingame_exit
l0570:
    jsr music_play
    ; end of irq
irq_handler_ingame_exit:
; DEBUG
!if DEBUG_IRQ_LEN > 0 {
    lda #$1b
    sta $900f
}
brk_handler:
    pla
    tay
    pla
    tax
    pla
    rti

hook_irq_handler_nop:
    sei
    lda #1
    sta flag_irq_handler_ingame_skip
    rts

; # main code and data (low)

read_joystick:
    lda flag_read_pl2
    bne read_pl2

read_pl1:
    lda $9111
    lsr
    tax
    and #$10    ; f
    sta read_pl1_smc
    txa
    lsr
    and #$07    ; udl
read_pl1_smc = * + 1
    ora #$00
    bit $9120   ; b7 = r
    bpl +
    ora #$08    ; r
+   eor #$ff
    and #$1f
    rts

read_pl2:
    lda flag_pl2_userport_joy
    bne read_pl2_userport

read_pl2_keyboard:
    ldx #$ef
    stx $9121
    lda $9120
    lsr
    and #3
    tax
    lda tbl_read_pl2_keyboard,x
    sta read_pl2_smc
    lda $9120
    and #$20
    bne +
    lda read_pl2_smc
    ora #$02
    sta read_pl2_smc
+
    ldx #$df
    stx $9121
    lda $9120
    and #$04
    bne +
    lda read_pl2_smc
    ora #$08
    sta read_pl2_smc
+
    ldx #$fd
    stx $9121
    lda $9120
    and #$08
    bne +
    lda read_pl2_smc
    ora #$10
    sta read_pl2_smc
+
    lda $9120
    and #$20
    ldx #$ff
    stx $9121
read_pl2_smc = * + 1
    lda #$00
    rts

tbl_read_pl2_keyboard:
!by $05, $04, $01, $00

read_pl2_userport:
    ldx #0
    lda #$80
    sta $9112
    sta $9110
    lda $9110
    stx $9112
    eor #$ff
    and #$1f
    rts

screen_disable:
    lda #(((vmatrix & $200) >> 2) | 0)
    sta $9002
wait_for_irq_x2:
    inc flag_wait_for_irq
-   lda flag_wait_for_irq
    bne -
    inc flag_wait_for_irq
-   lda flag_wait_for_irq
    bne -
    rts

screen_enable:
    jsr wait_for_irq_x2
    lda #(((vmatrix & $200) >> 2) | COLS)
    sta $9002
    rts

setup_screen_x:
    jsr screen_disable
    lda #CHAR_EMPTY
    jsr draw_clear
    cpx #0
    bne l0697
    lda #'H'
    sta screen_pos_logo - COLS * 2 + 2
    lda #'A'
    sta screen_pos_logo - COLS * 1 + 2
    ldy #6
-   lda text_trick,y
    sta screen_pos_logo,y
    dey
    bpl -
    ldy #10
-   lda text_line1,y
    sta screen_pos_line1,y
    lda text_line2,y
    sta screen_pos_line2,y
    lda text_line3,y
    sta screen_pos_line3,y
    lda text_line4,y
    sta screen_pos_line4,y
    dey
    bpl -

    ldy #18
-   lda text_pause,y
    sta screen_pos_pause,y
    lda text_quit,y
    sta screen_pos_quit,y
    lda text_press_fire,y
    sta screen_pos_press_fire,y
    dey
    bpl -

l0697:
    cpx #0
    bne l06a5
    lda flag_overtime
    beq l06a5
    rts
l06a5:
    jmp screen_enable

screen_pos_logo = vmatrix + COLS * 5 + 6
screen_pos_line1 = vmatrix + COLS * 9 + 1
screen_pos_num_players_num = screen_pos_line1 + 11
screen_pos_line2 = vmatrix + COLS * 10 + 1
screen_pos_p2_ctrl = screen_pos_line2 + 11
screen_pos_line3 = vmatrix + COLS * 11 + 1
screen_pos_skill = screen_pos_line3 + 11
screen_pos_line4 = vmatrix + COLS * 12 + 1
screen_pos_offon = screen_pos_line4 + 11
screen_pos_pause = vmatrix + COLS * 15 + 1
screen_pos_quit = vmatrix + COLS * 16 + 1
screen_pos_press_fire = vmatrix + COLS * 18 + 1
screen_pos_overtime = vmatrix + COLS * 10 + 3
text_trick:
!tx $80 | 81, " TRICK"
;    01234567890
text_line1:
!tx "   PLAYERS:"
text_line2:
!tx "P2 CONTROL:"
text_line3:
!tx "DIFFICULTY:"
text_line4:
!tx "     SOUND:"
;    0123456789012345678
text_pause:
!tx " RUN/STOP TO PAUSE "
text_quit:
!tx "  THEN Q TO QUIT   "
text_press_fire:
!tx "PRESS FIRE TO START"
text_move_joy_lr:
!tx "PICK SIDE WITH <  >"
text_p2_ctrl:
!tx "IJKL LSH"
!tx "USERPORT"
text_offon:
!tx "OFF"
text_on:
!tx "ON "
text_skill:
!tx "NOVICE"
!tx "NORMAL"
!tx "EXPERT"
text_skill_offset:
!by 0, 6, 12
text_overtime:
;    012345678901234
!tx "O V E R T I M E"

main_menu:
    jsr main_menu_print_settings
    jsr wait_15_frames
main_menu_input:
    jsr read_pl1
    sta temp_y
    bne l0937
    sta temp_x
    beq main_menu_input
l0937:
    lda temp_x
    bne main_menu_input
    lda temp_y
    sta temp_x
    and #$10    ; f
    beq l094f
    lda flag_num_players
    beq l094c
    ; Resetting this flag fixes
    ; 1. nonmoving goalie puck
    ; 2. shooting the wrong way
    ; which both occur on 2p after 1p on right side.
    lda #0
    sta flag_cpu_player
    rts
l094c:
    jmp l09ee
l094f:
    lda temp_y
    and #$08    ; r
    beq l0968
    lda flag_music
    eor #$01
    sta flag_music
    beq l0962
    lda #SOUND_VOLUME
l0962:
    sta $900e
    jmp main_menu
l0968:
    lda temp_y
    and #$01    ; u
    beq l097a
    lda flag_num_players
    eor #$01
    sta flag_num_players
    jmp main_menu
l097a:
    lda temp_y
    and #$04    ; l
    beq +
    inc flag_skill
    lda flag_skill
    cmp #$03
    bcc main_menu
    lda #$00
    sta flag_skill
    bne wait_15_frames
    jmp main_menu
+   lda temp_y
    and #$02    ; l
    beq main_menu_input
    lda flag_pl2_userport_joy
    eor #$01
    sta flag_pl2_userport_joy
    jmp main_menu

wait_15_frames:
    ldx #15
wait_x_frames:
    inc flag_wait_for_irq
-   lda flag_wait_for_irq
    bne -
    dex
    bne wait_x_frames
    rts

main_menu_print_settings:
    lda #<text_offon
    sta zpsrc
    lda #>text_offon
    sta zpsrch
    ldy #$00
    lda flag_music
    beq +
    ldy #$03
+   ldx #$00
-   lda (zpsrc),y
    sta screen_pos_offon,x
    iny
    inx
    cpx #$03
    bcc -
    ldx #'1'
    lda flag_num_players
    beq +
    inx
+   stx screen_pos_num_players_num
    lda #<text_skill
    sta zpsrc
    lda #>text_skill
    sta zpsrch
    ldx flag_skill
    ldy text_skill_offset,x
    ldx #$00
-   lda (zpsrc),y
    sta screen_pos_skill,x
    iny
    inx
    cpx #$06
    bcc -
    lda #<text_p2_ctrl
    sta zpsrc
    lda #>text_p2_ctrl
    sta zpsrch
    ldy #$00
    lda flag_pl2_userport_joy
    beq +
    ldy #$08
+   ldx #$00
-   lda (zpsrc),y
    sta screen_pos_p2_ctrl,x
    iny
    inx
    cpx #$08
    bcc -
    rts

l09ee:
    ldy #18
-   lda text_move_joy_lr,y
    sta screen_pos_press_fire,y
    dey
    bpl -
    lda #$00
    sta flag_read_pl2
-   jsr read_joystick
    and #$0c
    beq -
    and #$04
    bne +
    inc flag_read_pl2
+   lda flag_read_pl2
    sta flag_cpu_player
    rts

show_screen_overtime:
    jsr hook_irq_handler_nop
    cli
    jsr wait_for_irq_x2
    ldx #$00
    jsr setup_screen_x
    lda #CHAR_EMPTY
    jsr draw_clear
    ldy #14
-   lda text_overtime,y
    sta screen_pos_overtime,y
    dey
    bpl -
    jsr screen_enable
    ldy #$b4
--  inc flag_wait_for_irq
-   lda flag_wait_for_irq
    bne -
    dey
    bne --
    jsr hook_irq_handler_nop
    cli
    jsr wait_for_irq_x2
    ldx #$01
    jsr setup_screen_x
    jmp screen_enable


init_sprite_regs:
    jsr set_p2_sprptr
    ; FALL THROUGH

set_p1_sprptr:
    ldy p1_dir_i
    bmi l0b47
    lda puck_holder
    cmp #$01
    bne l0af4
    lda counter_hmm
    cmp #$03
    beq l0aed
    cmp #$01
    bne l0af4
    tya
    clc
    adc #$01
    jmp l0af1
l0aed:
    tya
    sec
    sbc #$01
l0af1:
    and #$07
    tay
l0af4:
    lda phase_hmm
    bne l0b18
    lda p1_dir_hmm
    bmi l0b18
    inc p1_anim_subframe
    lda p1_anim_subframe
    cmp #$08
    bcc l0b18
    lda #$00
    sta p1_anim_subframe
    inc p1_anim_frame
    lda p1_anim_frame
    and #$03
    sta p1_anim_frame
l0b18:
    lda tbl_spr_dir,y
    clc
    ldx p1_anim_frame
    adc tbl_anim_frame,x
    sta p1_anim_cooked
l0b47:
    rts

set_p2_sprptr:
    ldy p2_dir_i
    bmi l0bc0
    lda puck_holder
    cmp #$02
    bne l0b6d
    lda counter_hmm
    cmp #$03
    beq l0b66
    cmp #$01
    bne l0b6d
    tya
    clc
    adc #$01
    jmp l0b6a
l0b66:
    tya
    sec
    sbc #$01
l0b6a:
    and #$07
    tay
l0b6d:
    lda phase_hmm
    bne l0b91
    lda p2_dir_hmm
    bmi l0b91
    inc p2_anim_subframe
    lda p2_anim_subframe
    cmp #$08
    bcc l0b91
    lda #$00
    sta p2_anim_subframe
    inc p2_anim_frame
    lda p2_anim_frame
    and #$03
    sta p2_anim_frame
l0b91:
    lda tbl_spr_dir,y
    clc
    ldx p2_anim_frame
    adc tbl_anim_frame,x
    sta p2_anim_cooked
l0bc0:
    rts

tbl_anim_frame:
!by 0 << 6, 1 << 6, 2 << 6, 1 << 6
tbl_spr_dir:
!by 2 << 3, 4 << 3, 1 << 3, 7 << 3, 3 << 3, 5 << 3, 0 << 3, 6 << 3
tbl_spr_dir_base:
!by 2, 4, 1, 7, 3, 5, 0, 6

tbl_joy_to_dir:
!by $ff, $00, $04, $ff, $06, $07, $05, $ff, $02, $01, $03, $ff, $ff, $ff, $ff, $ff
tbl_dir_diff_x:
!by $00, $00, $00, $00, $ff, $ff, $ff, $00, $01, $01, $01, $00, $00, $00, $00, $00
tbl_dir_diff_y:
!by $00, $ff, $01, $00, $00, $ff, $01, $00, $00, $ff, $01, $00, $00, $00, $00, $00

move_p1_cond:
    lda phase_hmm
    beq move_p1
    rts
move_p1:
    lda p1_acc_x
    beq l0cb7
    bpl l0c98
    eor #$ff
    clc
    adc #$01
l0c98:
    clc
    adc p1_vel_x
    sta p1_vel_x
    cmp #5 ;px_lim_move_5
    bcc l0cb7
    sbc #5 ;px_lim_move_5
    sta p1_vel_x
    lda p1_acc_x
    bmi l0cb4
    inc p1_coord_x
    bne l0cb7
l0cb4:
    dec p1_coord_x
l0cb7:
    lda p1_acc_y
    beq l0ce1
    bpl l0cc3
    eor #$ff
    clc
    adc #$01
l0cc3:
    clc
    adc p1_vel_y
    sta p1_vel_y
    cmp #5 ;px_lim_move_5
    bcc l0ce1
    sbc #5 ;px_lim_move_5
    sta p1_vel_y
    lda p1_acc_y
    bmi l0cde
    inc p1_coord_y
    rts
l0cde:
    dec p1_coord_y
l0ce1:
    rts

get_input_p1_cond:
    lda phase_hmm
    beq get_input_p1
    rts
get_input_p1:
    lda #$00
    sta flag_read_pl2
    lda flag_num_players
    bne l0cfd
    lda flag_cpu_player
    beq l0cfd
    jsr cpu_player
    jmp l0d00
l0cfd:
    jsr read_joystick
l0d00:
    and #$0f
    tay
    ldx tbl_joy_to_dir,y
    stx p1_dir_hmm
    bpl l0d23
    ldx #$02
    lda p1_coord_x
    cmp #CO_PL_DIR_TH_X
    bcc l0d23
    ldx #$03
    lda p1_coord_y
    cmp #CO_PL_DIR_TH1_Y
    bcc l0d23
    dex
    cmp #CO_PL_DIR_TH2_Y
    bcc l0d23
    dex
l0d23:
    cpx p1_dir_i
    beq l0d37
    stx temp_y
    lda p1_dir_i
    sta temp_x
    jsr l0ded
    sta p1_dir_i
l0d37:
    inc p1_apply_counter
    lda p1_apply_counter
    cmp #$0f ;apply_forces_counter_top
    bcs apply_forces_p1
    rts
apply_forces_p1:
    lda #$00
    sta p1_apply_counter
    lda p1_acc_x
    beq l0d58
    bpl l0d55
    inc p1_acc_x
    jmp l0d58
l0d55:
    dec p1_acc_x
l0d58:
    lda p1_acc_y
    beq l0d68
    bpl l0d65
    inc p1_acc_y
    jmp l0d68
l0d65:
    dec p1_acc_y
l0d68:
    lda tbl_dir_diff_x,y
    beq l0da9
    bmi l0d8d
    inc p1_acc_x
    inc p1_acc_x
    lda #5 ;px_lim_move_5
    sta p1_vel_x
    lda p1_acc_x
    bmi l0da9
    cmp #5 ;px_lim_move_5
    bcc l0da9
    lda #5; px_lim_move_5
    sta p1_acc_x
    bne l0da9
l0d8d:
    dec p1_acc_x
    dec p1_acc_x
    lda #5 ;px_lim_move_5
    sta p1_vel_x
    lda p1_acc_x
    bpl l0da9
    cmp #-5 ;px_lim_move_m5
    bcs l0da9
    lda #-5 ;px_lim_move_m5
    sta p1_acc_x
l0da9:
    lda tbl_dir_diff_y,y
    beq l0dea
    bmi l0dce
    inc p1_acc_y
    inc p1_acc_y
    lda #5 ;px_lim_move_5
    sta p1_vel_y
    lda p1_acc_y
    bmi l0dea
    cmp #5 ;px_lim_move_5
    bcc l0dea
    lda #5 ;px_lim_move_5
    sta p1_acc_y
    bne l0dea
l0dce:
    dec p1_acc_y
    dec p1_acc_y
    lda #5 ;px_lim_move_5
    sta p1_vel_y
    lda p1_acc_y
    bpl l0dea
    cmp #-5 ;px_lim_move_m5
    bcs l0dea
    lda #-5 ;px_lim_move_m5
    sta p1_acc_y
l0dea:
    rts

l0deb:
!by $01
l0dec:
!by $00

l0ded:
    lda counter_hmm
    cmp #$04
    bcc l0e3c
    lda l0dec
    cmp l0deb
    bcc l0e39
    lda #$00
    sta l0dec
    sta l0e24
    lda temp_x
    sec
    sbc temp_y
    bcs l0e17
    ldx #$01
    stx l0e24
    eor #$ff
    clc
    adc #$01
l0e17:
    cmp #$04
    bcs l0e23
    lda l0e24
    eor #$01
    sta l0e24
l0e23:
l0e24 = * + 1
    lda #$ff
    bne l0e30
    inc temp_x
    lda temp_x
    and #$07
    rts
l0e30:
    dec temp_x
    lda temp_x
    and #$07
    rts
l0e39:
    inc l0dec
l0e3c:
    lda temp_x
    rts

; index 0 = no puck, 1 = with puck
p_coord_x_min:
!by CO_PLN_MIN_X, CO_PLP_MIN_X
p_coord_x_min_set:
!by CO_PLN_MIN_X, CO_PLP_MIN_X
p_coord_x_max:
!by CO_PLN_MAX_X + 1, CO_PLP_MAX_X + 1
p_coord_x_max_set:
!by CO_PLN_MAX_X, CO_PLP_MAX_X
p_coord_y_min:
!by CO_PLN_MIN_Y, CO_PLP_MIN_Y
p_coord_y_min_set:
!by CO_PLN_MIN_Y, CO_PLP_MIN_Y
p_coord_y_max:
!by CO_PLN_MAX_Y + 1, CO_PLP_MAX_Y + 1
p_coord_y_max_set:
!by CO_PLN_MAX_Y, CO_PLP_MAX_Y

check_coll_wall_p1:
    ldx pX_has_puck
    cpx #$02
    bcc l0e59
    ldx #$00
l0e59:
    lda p1_coord_x
    cmp p_coord_x_min,x
    bcs l0e66
    lda p_coord_x_min_set,x
    bne l0e6e
l0e66:
    cmp p_coord_x_max,x
    bcc l0e74
    lda p_coord_x_max_set,x
l0e6e:
    sta p1_coord_x
    jsr inv_p1_acc_x
l0e74:
    lda p1_coord_y
    cmp p_coord_y_min,x
    bcs l0e81
    lda p_coord_y_min_set,x
    bne l0e89
l0e81:
    cmp p_coord_y_max,x
    bcc l0e8f
    lda p_coord_y_max_set,x
l0e89:
    sta p1_coord_y
    jsr inv_p1_acc_y
l0e8f:
    rts

move_p2_cond:
    lda phase_hmm
    beq move_p2
    rts
move_p2:
    lda p2_acc_x
    beq l0ec1
    bpl l0ea2
    eor #$ff
    clc
    adc #$01
l0ea2:
    clc
    adc p2_vel_x
    sta p2_vel_x
    cmp #5 ;px_lim_move_5
    bcc l0ec1
    sbc #5 ;px_lim_move_5
    sta p2_vel_x
    lda p2_acc_x
    bmi l0ebe
    inc p2_coord_x
    bne l0ec1
l0ebe:
    dec p2_coord_x
l0ec1:
    lda p2_acc_y
    beq l0eeb
    bpl l0ecd
    eor #$ff
    clc
    adc #$01
l0ecd:
    clc
    adc p2_vel_y
    sta p2_vel_y
    cmp #5 ;px_lim_move_5
    bcc l0eeb
    sbc #5 ;px_lim_move_5
    sta p2_vel_y
    lda p2_acc_y
    bmi l0ee8
    inc p2_coord_y
    rts
l0ee8:
    dec p2_coord_y
l0eeb:
    rts

get_input_p2_cond:
    lda phase_hmm
    beq get_input_p2
    rts
get_input_p2:
    lda #$01
    sta flag_read_pl2
    lda flag_num_players
    bne l0f0a
    dec flag_read_pl2
    lda flag_cpu_player
    bne l0f0a
    jsr cpu_player
    jmp l0f0d
l0f0a:
    jsr read_joystick
l0f0d:
    and #$0f
    tay
    ldx tbl_joy_to_dir,y
    stx p2_dir_hmm
    bpl l0f30
    ldx #$06
    lda p2_coord_x
    cmp #CO_PL_DIR_TH_X
    bcs l0f30
    ldx #$05
    lda p2_coord_y
    cmp #CO_PL_DIR_TH1_Y
    bcc l0f30
    inx
    cmp #CO_PL_DIR_TH2_Y
    bcc l0f30
    inx
l0f30:
    cpx p2_dir_i
    beq l0f44
    stx temp_y
    lda p2_dir_i
    sta temp_x
    jsr l0ded
    sta p2_dir_i
l0f44:
    inc p2_apply_counter
    lda p2_apply_counter
    cmp #$0f ;apply_forces_counter_top
    bcs apply_forces_p2
    rts
apply_forces_p2:
    lda #$00
    sta p2_apply_counter
    lda p2_acc_x
    beq l0f65
    bpl l0f62
    inc p2_acc_x
    jmp l0f65
l0f62:
    dec p2_acc_x
l0f65:
    lda p2_acc_y
    beq l0f75
    bpl l0f72
    inc p2_acc_y
    jmp l0f75
l0f72:
    dec p2_acc_y
l0f75:
    lda tbl_dir_diff_x,y
    beq l0fb6
    bmi l0f9a
    inc p2_acc_x
    inc p2_acc_x
    lda #5 ;px_lim_move_5
    sta p2_vel_x
    lda p2_acc_x
    bmi l0fb6
    cmp #5 ;px_lim_move_5
    bcc l0fb6
    lda #5 ;px_lim_move_5
    sta p2_acc_x
    bne l0fb6
l0f9a:
    dec p2_acc_x
    dec p2_acc_x
    lda #5 ;px_lim_move_5
    sta p2_vel_x
    lda p2_acc_x
    bpl l0fb6
    cmp #-5 ;px_lim_move_m5
    bcs l0fb6
    lda #-5 ;px_lim_move_m5
    sta p2_acc_x
l0fb6:
    lda tbl_dir_diff_y,y
    beq l0ff7
    bmi l0fdb
    inc p2_acc_y
    inc p2_acc_y
    lda #5 ;px_lim_move_5
    sta p2_vel_y
    lda p2_acc_y
    bmi l0ff7
    cmp #5 ;px_lim_move_5
    bcc l0ff7
    lda #5 ;px_lim_move_5
    sta p2_acc_y
    bne l0ff7
l0fdb:
    dec p2_acc_y
    dec p2_acc_y
    lda #5 ;px_lim_move_5
    sta p2_vel_y
    lda p2_acc_y
    bpl l0ff7
    cmp #-5 ;px_lim_move_m5
    bcs l0ff7
    lda #-5 ;px_lim_move_m5
    sta p2_acc_y
l0ff7:
    rts

check_coll_wall_p2:
    ldx pX_has_puck
    beq l0fff
    ldx #$01
l0fff:
    lda p2_coord_x
    cmp p_coord_x_min,x
    bcs l100c
    lda p_coord_x_min_set,x
    bne l1014
l100c:
    cmp p_coord_x_max,x
    bcc l101a
    lda p_coord_x_max_set,x
l1014:
    sta p2_coord_x
    jsr inv_p2_acc_x
l101a:
    lda p2_coord_y
    cmp p_coord_y_min,x
    bcs l1027
    lda p_coord_y_min_set,x
    bne l102f
l1027:
    cmp p_coord_y_max,x
    bcc l1035
    lda p_coord_y_max_set,x
l102f:
    sta p2_coord_y
    jsr inv_p2_acc_y
l1035:
    rts

move_goalies:
    lda phase_hmm
    bne l109b
!if 0 {
    ; This fixes the bug of nonmoving goalie,
    ; which happens after playing 1p game on right and then 2p game.
    ; Resetting flag_cpu_player also fixes this.
    lda flag_num_players
    bne +
}
    lda flag_cpu_player
    bne l106b
+   lda p1_dir_hmm
    bmi l106b
    cmp #$02
    beq l106b
    bcc l1061
    cmp #$06
    beq l106b
    bcs l1061
    lda g1_coord_y
    cmp #CO_GOA_MAX_Y
    bcs l106b
    inc g1_coord_y
    bne l106b
l1061:
    lda g1_coord_y
    cmp #CO_GOA_MIN_Y
    bcc l106b
    dec g1_coord_y
l106b:
    lda flag_num_players
    bne l1075
    lda flag_cpu_player
    beq l109b
l1075:
    lda p2_dir_hmm
    bmi l109b
    cmp #$02
    beq l109b
    bcc l1091
    cmp #$06
    beq l109b
    bcs l1091
    lda g2_coord_y
    cmp #CO_GOA_MAX_Y
    bcs l109b
    inc g2_coord_y
    rts
l1091:
    lda g2_coord_y
    cmp #CO_GOA_MIN_Y
    bcc l109b
    dec g2_coord_y
l109b:
    rts

tbl_puck_start_y:
!by CO_PUCK_START_Y_0, CO_PUCK_START_Y_1
tbl_puck_start_acc_y:
!by -4, 4

start_puck:
    lda #CO_PUCK_START_X
    sta puck_coord_x
    ldx #$00
    lda $9114   ; timer low (random)
    bmi l10b2
    inx
l10b2:
    lda tbl_puck_start_y,x
    sta puck_coord_y
    stx l044b
    lda tbl_puck_start_acc_y,x
    sta puck_acc_y
    lda #$00
    sta puck_acc_x
    sta puck_free_counter
    sta pX_has_puck
    rts

handle_puck:
    lda phase_hmm
    beq l10d4
    rts
l10d4:
    lda pX_has_puck
    beq l10df
    sta puck_holder
    jmp move_puck_px
l10df:
    ldy counter_hmm
    cpy #$04
    bcs move_puck
    inc counter_hmm_sub
    lda counter_hmm_sub
    cmp #$08
    bcc l10f8
    lda #$00
    sta counter_hmm_sub
    inc counter_hmm
l10f8:
    cpy #$03
    bcs move_puck
    lda puck_holder
    jmp move_puck_px
move_puck:
    lda puck_acc_x
    beq l112d
    bpl l110e
    eor #$ff
    clc
    adc #$01
l110e:
    clc
    adc puck_vel_x
    sta puck_vel_x
    cmp #8 ;puck_lim_move_8
    bcc l112d
    sbc #8 ;puck_lim_move_8
    sta puck_vel_x
    lda puck_acc_x
    bmi l112a
    inc puck_coord_x
    bne l112d
l112a:
    dec puck_coord_x
l112d:
    lda puck_acc_y
    beq l1158
    bpl l1139
    eor #$ff
    clc
    adc #$01
l1139:
    clc
    adc puck_vel_y
    sta puck_vel_y
    cmp #8 ;puck_lim_move_8
    bcc l1158
    sbc #8 ;puck_lim_move_8
    sta puck_vel_y
    lda puck_acc_y
    bmi l1155
    inc puck_coord_y
    bne l1158
l1155:
    dec puck_coord_y
l1158:
    rts

; a = player with puck
move_puck_px:
    cmp #$01
    bne move_puck_p2
    lda p1_dir_i
    ldx p1_coord_x
    ldy p1_coord_y
    bne move_puck_px_do
move_puck_p2:
    cmp #$02
    bne l1190
    lda p2_dir_i
    ldx p2_coord_x
    ldy p2_coord_y
move_puck_px_do:
    stx temp_x
    sty temp_y
    tay
    lda temp_x
    clc
    adc tbl_puck_dir_x_offset,y
    sta puck_coord_x
    lda temp_y
    clc
    adc tbl_puck_dir_y_offset,y
    sta puck_coord_y
l1190:
    rts

!if DEBUG_IRQ_LEN = 0 {
tbl_puck_dir_x_offset:
!by 9, 11, 11, 11, 1, -1, -1, -1
tbl_puck_dir_y_offset:
!by -4, -1, 14, 19, 20, 18, 3, -2
}

move_free_puck:
    lda phase_hmm
    bne l11e8
    lda pX_has_puck
    bne l11e8
    inc puck_free_counter
    lda puck_free_counter
    cmp #80
    bcc l11e8
    lda #$00
    sta puck_free_counter
; reduce puck acc to +-1
    lda puck_acc_x
    cmp #$02
    bcc l11d1
    cmp #$ff
    beq l11d1
    bmi l11ce
    dec puck_acc_x
    jmp l11d1
l11ce:
    inc puck_acc_x
l11d1:
    lda puck_acc_y
    cmp #$02
    bcc l11e7
    cmp #$ff
    beq l11e7
    bmi l11e4
    dec puck_acc_y
    jmp l11e7
l11e4:
    inc puck_acc_y
l11e7:
    rts
l11e8:
    rts

tbl_limit_puck_x_min:
!by CO_PUCKN_MIN_X, CO_PUCKG_MIN_X
tbl_limit_puck_x_min_set:
!by CO_PUCKN_MIN_X, CO_PUCKG_MIN_X
tbl_limit_puck_x_max:
!by CO_PUCKN_MAX_X + 1, CO_PUCKG_MAX_X + 1
tbl_limit_puck_x_max_set:
!by CO_PUCKN_MAX_X, CO_PUCKG_MAX_X

check_puck_limits:
    lda #$00
    sta temp_x
    lda pX_has_puck
    beq l11fe
    jmp l1286
l11fe:
    lda puck_coord_y
    cmp #CO_PUCK_MIN_Y
    bcs l1209
    lda #CO_PUCK_MIN_Y
    bne l120f
l1209:
    cmp #CO_PUCK_MAX_Y + 1
    bcc l1223
    lda #CO_PUCK_MAX_Y
l120f:
    sta puck_coord_y
    lda puck_acc_y
    eor #$ff
    clc
    adc #$01
    sta puck_acc_y
    lda #8 ;puck_lim_move_8
    sta puck_vel_y
l1223:
    ldx #$00
    lda puck_coord_y
    cmp #CO_PUCK_IN_G_MIN_Y
    bcc l1231
    cmp #CO_PUCK_IN_G_MAX_Y + 1
    bcs l1231
    inx
l1231:
    lda puck_coord_x
    cmp tbl_limit_puck_x_min,x
    bcs l123e
    lda tbl_limit_puck_x_min_set,x
    bne l1246
l123e:
    cmp tbl_limit_puck_x_max,x
    bcc l1286
    lda tbl_limit_puck_x_max_set,x
l1246:
    sta puck_coord_x
    txa
    beq l1257
    lda #$00
    sta l044a
    lda #$02
    sta phase_hmm
    rts
l1257:
    lda temp_x
    beq l126f
    ldy #$00
    lda puck_coord_x
    bpl l1264
    iny
l1264:
    sed
    lda saves_p1,y
    clc
    adc #$01
    sta saves_p1,y
    cld
l126f:
    lda puck_acc_x
    eor #$ff
    clc
    adc #$01
    sta puck_acc_x
    lda #8 ;puck_lim_move_8
    sta puck_vel_x
    lda #$04
    jsr music_init
l1285:
    rts
l1286:
    inc temp_x
    lda puck_coord_x
    cmp #CO_PUCK_L_MIN_X
    bcc l1285
    cmp #CO_PUCK_L_MAX_X
    bcs l12b3
    lda g1_coord_y
    clc
    adc #$03
    cmp puck_coord_y
    bcs l1285
    adc #14
    cmp puck_coord_y
    bcc l1285
    ldy #$04
    lda pX_has_puck
    bne l12dc
    lda puck_acc_x
    bmi l1257
l12b2:
    rts
l12b3:
    cmp #CO_PUCK_R_MIN_X
    bcc l1285
    cmp #CO_PUCK_R_MAX_X
    bcs l1285
    lda g2_coord_y
    clc
    adc #$03
    cmp puck_coord_y
    bcs l1285
    adc #$0e
    cmp puck_coord_y
    bcc l1285
    ldy #$fc
    lda pX_has_puck
    bne l12dc
    lda puck_acc_x
    bmi l12b2
    jmp l1257
l12dc:
    sty puck_acc_x
    lda #8 ;puck_lim_move_8
    sta puck_vel_x
    ldx p1_acc_x
    lda pX_has_puck
    cmp #$01
    beq l12f2
    ldx p2_acc_x
l12f2:
    cpx #$00
    beq l1304
    tya
    bpl l12fe
    txa
    bmi l1301
    bpl l1304
l12fe:
    txa
    bmi l1304
l1301:
    stx puck_acc_x
l1304:
    lda $9114   ; timer low (random)
    and #$03
    tax
    lda tbl_rand_puck_acc,x
    sta puck_acc_y
    lda #$00
    sta pX_has_puck
    lda #$0e
    sta timeout_check_collider_puck_p1
    sta timeout_check_collider_puck_p2
    rts

tbl_rand_puck_acc:
!by $03, $05, $02, $04

tbl_mask_coll:
!by $03, $02, $0a, $08, $0c, $04, $05, $01

check_collider_foo3:
    lda phase_hmm
    beq l1332
    rts
l1332:
    lda timeout_check_collider_puck_p1
    beq l133a
    dec timeout_check_collider_puck_p1
l133a:
    lda timeout_check_collider_puck_p2
    beq l1342
    dec timeout_check_collider_puck_p2
l1342:
    lda counter_hmm
    cmp #$04
    bcc l13a2
    lda timeout_check_collider_puck_p1
    bne l1376
    lda pX_has_puck
    cmp #$01
    beq l1376
    ldx p1_coord_x
    ldy p1_coord_y
    jsr check_coll_puck
    ldy p1_dir_i
    and tbl_mask_coll,y
    beq l1376
    lda #$01
    sta pX_has_puck
    lda #$08
    sta timeout_check_collider_puck_p2
    lda #$03
    jsr music_init
    rts
l1376:
    lda timeout_check_collider_puck_p2
    bne l13a2
    lda pX_has_puck
    cmp #$02
    beq l13a2
    ldx p2_coord_x
    ldy p2_coord_y
    jsr check_coll_puck
    ldy p2_dir_i
    and tbl_mask_coll,y
    beq l13a2
    lda #$02
    sta pX_has_puck
    lda #$08
    sta timeout_check_collider_puck_p1
    lda #$03
    jsr music_init
l13a2:
    rts

Main_low_end = *

!if * > charset {
!error "code low under charset"
!error * - charset
}

; ###############

* = charset

!src "gfx_chars.asm"

; # code

!zn Phase2_Init {   ; all within the zone can be overwritten after run once

; # p2 code
init_phase2:
    ; draw static gfx
    ldx #COLS - 1
-   lda text_top_line,x
    sta vmatrix,x
    lda #CHAR_WALL_U
    sta vmatrix + COLS * 1,x
    lda #CHAR_WALL_D
    sta vmatrix + COLS * (ROWS - 1),x
    dex
    bpl -

    ldx #1
-   lda p2_tbl_y_char_l,x
    sta zpdst
    lda p2_tbl_y_char_h,x
    sta zpdsth
    lda text_column_l - 1,x
    ldy #0
    sta (zpdst),y
    ldy #COLS - 1
    lda text_column_r - 1,x
    sta (zpdst),y
    inx
    cpx #ROWS
    bne -

; ### build speedcode

    lda #<draw_clear
    sta zpdst
    lda #>draw_clear
    sta zpdsth

    lda #<(vmatrix + 2 * COLS + 1)
    sta zpsrc
    lda #>(vmatrix + 2 * COLS + 1)
    sta zpsrch

    ldx #(AREA_H - 2)
    stx Temp_lines

--
    ldx #(AREA_W - 2)

    ldy #0
-   lda #OP_STA_ABS
    sta (zpdst),y
    inc zpdst
    bne +
    inc zpdsth
+   lda zpsrc
    sta (zpdst),y
    inc zpdst
    bne +
    inc zpdsth
+   lda zpsrch
    sta (zpdst),y
    inc zpdst
    bne +
    inc zpdsth
+   inc zpsrc
    bne +
    inc zpsrch
+
    dex
    bne -
+   inc zpsrc
    bne +
    inc zpsrch
+   inc zpsrc
    bne +
    inc zpsrch
+
    dec Temp_lines
    bne --

    lda #OP_RTS
    sta (zpdst),y

; ### build y char LUT
; (stolen from Crossroads)

    lda #0
    ldx #>vmatrix
    tay
init_tbl_screen_line_addr:
-   sta tbl_y_char_l,y
    pha
    txa
    sta tbl_y_char_h,y
    iny
    cpy #ROWS
    beq +
    pla
    clc
    adc #COLS
    bcc -
    inx
    bcs -
+   pla

; ###

    ; clear variables
    lda #0
    ldx #2
-   sta $0,x
    inx
    bne -

    ; detect pal/ntsc
    lda $ede4
    and #$01
    tax ; 0 -> pal, 1 -> ntsc
    beq +
    lda #5
    sta $9000
    lda #26
    sta $9001
+
    lda #$40
    sta $912b       ; enable Timer A free run on VIA 1

    lda gamefw_tv_tbl_raster,x
    pha
    lda gamefw_tv_tbl_timer_l,x
    tay
    lda gamefw_tv_tbl_timer_h,x
    tax
    ; synchronise with the screen
    pla
-   cmp $9004
    beq -
-   cmp $9004
    bne -

    sty $9126
    stx $9125

    lda #$c0
    cli
    sta $912e

    jmp entry2

; # p2 data

; tbl: index 0 -> pal, 1 -> ntsc
gamefw_tv_tbl_raster:
!by 111, 100
gamefw_tv_tbl_timer_l:
!by <(312 * 71 - 2), <(261 * 65 - 2)
gamefw_tv_tbl_timer_h:
!by >(312 * 71 - 2), >(261 * 65 - 2)

text_top_line:
;    012345678901234567890
!tx "G00 S00 T0:00 S00 G00"
Offs_text_g1 = 1
Offs_text_g2 = 19
Offs_text_s1 = 5
Offs_text_s2 = 15
Offs_text_tm = 9
Offs_text_ts = 11

text_column_l:
!by CHAR_WALL_UL
!for .i, 6 {
!by CHAR_WALL_L
}
!by CHAR_GOAL_LU
!by CHAR_GOAL_LM
!by CHAR_GOAL_LM
!by CHAR_GOAL_LM
!by CHAR_GOAL_LM
!by CHAR_GOAL_LD
!for .i, 6 {
!by CHAR_WALL_L
}
!by CHAR_WALL_DL

text_column_r:
!by CHAR_WALL_UR
!for .i, 6 {
!by CHAR_WALL_R
}
!by CHAR_GOAL_RU
!by CHAR_GOAL_RM
!by CHAR_GOAL_RM
!by CHAR_GOAL_RM
!by CHAR_GOAL_RM
!by CHAR_GOAL_RD
!for .i, 6 {
!by CHAR_WALL_R
}
!by CHAR_WALL_DR

p2_tbl_y_char_l:
!for .i, ROWS {
!by <(vmatrix + (.i - 1) * COLS)
}

p2_tbl_y_char_h:
!for .i, ROWS {
!by >(vmatrix + (.i - 1) * COLS)
}

IP2_End = *
} ; !zn Phase2_Init

; ###############

; # main code and data (high)

!if (* < (charset + CHAR_BASE_GFX * 8 + $200)) {
!warn "draw code can write over main, overlap below, fixing"
!warn (charset + CHAR_BASE_GFX * 8 + $200) - *
* = (charset + CHAR_BASE_GFX * 8 + $200) + $100
}

!if DEBUG_IRQ_LEN > 0 {
tbl_puck_dir_x_offset:
!by 9, 11, 11, 11, 1, -1, -1, -1
tbl_puck_dir_y_offset:
!by -4, -1, 14, 19, 20, 18, 3, -2
}

check_coll_puck:
    stx temp_x
    sty temp_y
    lda puck_coord_x
    sec
    sbc temp_x
    bcs l13b9
check_coll_puck_no_coll:
    lda #$00
    rts
l13b9:
    sta temp_foo3_minus_x
    lda puck_coord_y
    sec
    sbc temp_y
    bcc check_coll_puck_no_coll
    sta temp_foo3_minus_y
    cmp #$12 ;const_coll_diff_y_12
    bcs check_coll_puck_no_coll
    lda temp_foo3_minus_x
    cmp #$10 ;const_coll_diff_x_10
    bcs check_coll_puck_no_coll
    lda temp_foo3_minus_y
    cmp #$09 ;const_coll_diff_y_09
    bcs l13e1
    lda #$01
    bne l13e3
l13e1:
    lda #$04
l13e3:
    sta temp_hmm
    lda temp_foo3_minus_x
    cmp #$08 ;const_coll_diff_x_08
    bcc l13f1
    asl temp_hmm
l13f1:
    lda temp_hmm
    rts

tbl_dir_to_puck_acc_x:
!by $00, $06, $06, $06, $00, $fa, $fa, $fa
tbl_dir_to_puck_acc_y:
!by $fa, $fa, $00, $06, $06, $06, $00, $fa

check_shoot_puck:
    ldx pX_has_puck
    bne l140b
    rts
l140b:
    dex
    stx flag_read_pl2
    lda flag_num_players
    bne l1424
    sta flag_read_pl2
    cpx flag_cpu_player
    beq l1424
    jsr cpu_check_shoot_puck
    inc flag_read_pl2
    bne l1427
l1424:
    jsr read_joystick
l1427:
    and #$10
    bne l142c
    rts
l142c:
    lda flag_cpu_player
    beq l1439
    lda flag_read_pl2
    eor #$01
    sta flag_read_pl2
l1439:
    lda #$00
    sta counter_hmm_sub
    sta counter_hmm
    lda #$02
    jsr music_init
    lda flag_read_pl2
    bne l14b6
    ldy p1_dir_i
    lda p1_acc_x
    clc
    adc tbl_dir_to_puck_acc_x,y
    bmi l1470
    cmp #6 ;px_lim_move_6
    bcc l145f
    lda #6 ;px_lim_move_6
l145f:
    sta puck_acc_x
    lda #8 ;puck_lim_move_8
    sta puck_vel_x
    lda #$00
    sta puck_free_counter
    jmp l147a
l1470:
    cmp #-6 ;px_lim_move_m6
    bcs l145f
    lda #-6 ;px_lim_move_m6
    bne l145f  ; bra
l147a:
    lda p1_acc_y
    clc
    adc tbl_dir_to_puck_acc_y,y
    bmi l149c
    cmp #6 ;px_lim_move_6
    bcc l148b
    lda #6 ;px_lim_move_6
l148b:
    sta puck_acc_y
    lda #8 ;puck_lim_move_8
    sta puck_vel_y
    lda #$00
    sta puck_free_counter
    jmp l14a6
l149c:
    cmp #-6 ;px_lim_move_m6
    bcs l148b
    lda #-6 ;px_lim_move_m6
    bne l148b
l14a6:
    lda #$00
    sta pX_has_puck
    lda #$18
    sta timeout_check_collider_puck_p1
    lda #$02
    jsr music_init
    rts
l14b6:
    ldy p2_dir_i
    lda p2_acc_x
    clc
    adc tbl_dir_to_puck_acc_x,y
    bmi l14db
    cmp #6 ;px_lim_move_6
    bcc l14ca
    lda #6 ;px_lim_move_6
l14ca:
    sta puck_acc_x
    lda #8 ;puck_lim_move_8
    sta puck_vel_x
    lda #$00
    sta puck_free_counter
    jmp l14e5
l14db:
    cmp #-6 ;px_lim_move_m6
    bcs l14ca
    lda #-6 ;px_lim_move_m6
    bne l14ca
l14e5:
    lda p2_acc_y
    clc
    adc tbl_dir_to_puck_acc_y,y
    bmi l1507
    cmp #6 ;px_lim_move_6
    bcc l14f6
    lda #6 ;px_lim_move_6
l14f6:
    sta puck_acc_y
    lda #8 ;puck_lim_move_8
    sta puck_vel_y
    lda #$00
    sta puck_free_counter
    jmp l1511
l1507:
    cmp #-6 ;px_lim_move_m6
    bcs l14f6
    lda #-6 ;px_lim_move_m6
    bne l14f6
l1511:
    lda #$00
    sta pX_has_puck
    lda #$18
    sta timeout_check_collider_puck_p2
    lda #$02
    jsr music_init
    rts

cond_check_coll_p1_p2:
    lda phase_hmm
    beq check_coll_p1_p2_timeout
    rts
check_coll_p1_p2_timeout:
    lda timeout_check_coll_p1_p2
    beq check_coll_p1_p2
    dec timeout_check_coll_p1_p2
    rts
check_coll_p1_p2:
    lda p2_coord_x
    clc
    adc #$0c
    sta temp_x
    lda p1_coord_x
    clc
    adc #$0c
    sec
    sbc temp_x
    bcs l154c
    eor #$ff
    adc #$01
l154c:
    cmp #5 ;px_coll_diff_x
    bcs l1591
    lda p2_coord_y
    clc
    adc #$0b
    sta temp_x
    lda p1_coord_y
    clc
    adc #$0b
    sec
    sbc temp_x
    bcs l156a
    eor #$ff
    adc #$01
l156a:
    cmp #10 ;px_coll_diff_y
    bcs l1591
    ldy p1_acc_x
    lda p2_acc_x
    sty p2_acc_x
    sta p1_acc_x
    ldy p1_acc_y
    lda p2_acc_y
    sty p2_acc_y
    sta p1_acc_y
    lda #$05
    sta timeout_check_coll_p1_p2
    lda #$03
    jsr music_init
l1591:
    rts

inv_p1_acc_x:
    lda p1_acc_x
    eor #$ff
    clc
    adc #$01
    sta p1_acc_x
    lda #5 ;px_lim_move_5
    sta p1_vel_x
    lda #$03
    jsr music_init
    rts

inv_p1_acc_y:
    lda p1_acc_y
    eor #$ff
    clc
    adc #$01
    sta p1_acc_y
    lda #5 ;px_lim_move_5
    sta p1_vel_y
    lda #$03
    jsr music_init
    rts

inv_p2_acc_x:
    lda p2_acc_x
    eor #$ff
    clc
    adc #$01
    sta p2_acc_x
    lda #5 ;px_lim_move_5
    sta p2_vel_x
    lda #$03
    jsr music_init
    rts

inv_p2_acc_y:
    lda p2_acc_y
    eor #$ff
    clc
    adc #$01
    sta p2_acc_y
    lda #5 ;px_lim_move_5
    sta p2_vel_y
    lda #$03
    jsr music_init
    rts

l15ee:
    lda phase_hmm
    beq l1606
    cmp #$01
    bne l1607
; phase_hmm == 1
    inc l044a
    lda l044a
    cmp #71
    bcc l1606
    lda #$00
    sta phase_hmm
; phase_hmm == 0
l1606:
    rts
l1607:
    cmp #$02
    bne l1645
; phase_hmm == 2
    inc l044a
    lda l044a
    cmp #2
    bcs l1637
    lda #$06
    jsr music_init
    lda puck_coord_x
    bmi l162b
    sed
    lda score_bcd_p2
    clc
    adc #$01
    sta score_bcd_p2
    cld
    rts
l162b:
    sed
    lda score_bcd_p1
    clc
    adc #$01
    sta score_bcd_p1
    cld
    rts
l1637:
    cmp #176
    bcc l1606
    lda flag_overtime
    bne l166c
    pla
    pla
    jmp init_music_and_game_loop

; phase_hmm >= 3
l1645:
    inc l044a
    lda l044a
    cmp #80
    bcc l1606
    lda flag_overtime
    bne l166c
    lda score_bcd_p1
    cmp score_bcd_p2
    bne l166c
    inc flag_overtime
    lda #$01
    sta timer_m
    jsr show_screen_overtime
    pla
    pla
    jmp init_game_loop
l166c:
;    jsr init_cleaner
    pla
    pla
    jmp entry3

draw_scores:
    lda score_bcd_p1
    ldy #Offs_text_g1
    jsr draw_num
    lda score_bcd_p2
    ldy #Offs_text_g2
    jsr draw_num
draw_saves:
    lda saves_p1
    ldy #Offs_text_s1
    jsr draw_num
    lda saves_p2
    ldy #Offs_text_s2
    jsr draw_num
    rts

timer_init_3_00_draw:
    lda #$03
    sta timer_m
    lda #$00
    sta timer_s

draw_timer:
    lda timer_m
    ldy #Offs_text_tm
    jsr draw_num1
    lda timer_s
    ldy #Offs_text_ts
    jsr draw_num
    rts

timer_tick:
    lda phase_hmm
    bne timer_tick_done
    inc timer_sub_60
    lda timer_sub_60
    cmp #60
    bcc timer_tick_done
    lda #$00
    sta timer_sub_60
    lda timer_s
    beq timer_tick_sub_min
    sed
    lda timer_s
    sec
    sbc #$01
    sta timer_s
    cld
    rts
timer_tick_sub_min:
    lda timer_m
    beq timer_zero
    dec timer_m
    lda #$59
    sta timer_s
timer_tick_done:
    rts
timer_zero:
    sta l044a
    lda #$03
    sta phase_hmm
    rts

; a = number
; y = column
draw_num:
    sta draw_num_smc
    lsr
    lsr
    lsr
    lsr
    tax
    lda hexchars,x
    sta vmatrix,y
    iny
draw_num_smc = * + 1
    lda #$00 ;temp_num_bcd
draw_num1:
    and #$f
    tax
    lda hexchars,x
    sta vmatrix,y
    rts

hexchars:
!tx "0123456789ABCDEF"

check_pause_key:
    lda #$00
    sta flag_skip_irq_logic2
    ldx #$fe
    stx $9121
    lda $9120
    and #$08  ; run/stop
    bne laa26
    inc flag_skip_irq_logic2
    lda #$00
    sta $900e
    jsr delay2
wait_pause_not_pressed:
    lda $9120
    and #$08  ; run/stop
    beq wait_pause_not_pressed
    jsr delay2
wait_abort_pause_pressed:
    lda $9120
    and #$40  ; q
    beq abort_was_pressed
    lda $9120
    and #$08  ; run/stop
    bne wait_abort_pause_pressed
    jsr delay2
wait_pause_not_pressed_2:
    lda $9120
    and #$08  ; run/stop
    beq wait_pause_not_pressed_2
    jsr delay2
    lda flag_music
    beq laa26
    lda #SOUND_VOLUME
    sta $900e
laa26:
    ldx #$ff
    stx $9121
    rts
abort_was_pressed:
    ldx #$ff
    stx $9121
    lda flag_music
    beq laa40
    lda #SOUND_VOLUME
    sta $900e
laa40:
    pla
    pla
    jmp entry3

delay2:
    ldx #$00
laa47:
    nop
    nop
    nop
    dex
    bne laa47
    rts

cpu_player:
    lda flag_cpu_player
    bne laa82
    lda p1_coord_x
    clc
    adc #$0c
    sta temp_x
    lda p1_coord_y
    clc
    adc #$0a
    sta temp_y
    lda p2_coord_x
    sta temp_foo3_minus_x
    lda p2_coord_y
    clc
    adc #$0a
    sta temp_foo3_minus_y
    lda #$91
    sta temp_hmm
    lda g2_coord_y
    sta l0415
    jmp laaae
laa82:
    lda p2_coord_x
    sta temp_x
    lda p2_coord_y
    clc
    adc #$0a
    sta temp_y
    lda p1_coord_x
    clc
    adc #$0c
    sta temp_foo3_minus_x
    lda p1_coord_y
    clc
    adc #$0a
    sta temp_foo3_minus_y
    lda #$0d
    sta temp_hmm
    lda g1_coord_y
    sta l0415
laaae:
    lda temp_hmm
    sec
    sbc puck_coord_x
    bcs laabb
    eor #$ff
    adc #$01
laabb:
    ldy cpu_skill_hmm_i
    cmp lac0e,y
    bcs laad5
    inc lac09
    lda lac09
    cmp #$02
    bcc laad5
    lda #$00
    sta lac09
    jsr labc6
laad5:
    lda flag_cpu_player
    clc
    adc #$01
    eor #$03
    cmp pX_has_puck
    bne laae5
    jmp lab4c
laae5:
    ldy cpu_skill_hmm_i
    inc lac0a
    lda lac0a
    cmp lac1e,y
    bcs laaf7
    lda lac0b
    rts
laaf7:
    lda #$00
    sta l0416
    sta lac0a
    lda temp_foo3_minus_x
    sec
    sbc puck_coord_x
    bcc lab16
    cmp #$05
    bcc lab22
    lda l0416
    ora #$04
    sta l0416
    bne lab22
lab16:
    cmp #$fc
    bcs lab22
    lda l0416
    ora #$08
    sta l0416
lab22:
    lda temp_foo3_minus_y
    sec
    sbc puck_coord_y
    bcc lab39
    cmp #$05
    bcc lab45
    lda l0416
    ora #$01
    sta l0416
    bne lab45
lab39:
    cmp #$fc
    bcs lab45
    lda l0416
    ora #$02
    sta l0416
lab45:
    lda l0416
    sta lac0b
    rts
lab4c:
    lda cpu_shoot_delay
    beq lab54
    lda #$00
    rts
lab54:
    ldy cpu_skill_hmm_i
    inc lac0a
    lda lac0a
    cmp lac2e,y
    bcs lab66
    lda lac0b
    rts
lab66:
    lda #$00
    sta l0416
    sta lac0a
    ldx #$09
    lda flag_cpu_player
    beq lab77
    ldx #$95
lab77:
    stx temp_hmm
    lda temp_foo3_minus_x
    sec
    sbc temp_hmm
    bcc lab91
    cmp #$05
    bcc lab9d
    lda l0416
    ora #$04
    sta l0416
    bne lab9d
lab91:
    cmp #$fc
    bcs lab9d
    lda l0416
    ora #$08
    sta l0416
lab9d:
    lda temp_foo3_minus_y
    sec
    sbc #$64
    bcc labb3
    cmp #$05
    bcc labbf
    lda l0416
    ora #$01
    sta l0416
    bne labbf
labb3:
    cmp #$fc
    bcs labbf
    lda l0416
    ora #$02
    sta l0416
labbf:
    lda l0416
    sta lac0b
    rts
labc6:
    lda l0415
    clc
    adc #$0a
    cmp puck_coord_y
    beq labe2
    bcc labee
    lda flag_cpu_player
    beq labe3
    lda g1_coord_y
    cmp #CO_GOA_MIN_Y
    bcc labe2
    dec g1_coord_y
labe2:
    rts
labe3:
    lda g2_coord_y
    cmp #CO_GOA_MIN_Y
    bcc labe2
    dec g2_coord_y
    rts
labee:
    lda flag_cpu_player
    beq labfe
    lda g1_coord_y
    cmp #CO_GOA_MAX_Y
    bcs labe2
    inc g1_coord_y
    rts
labfe:
    lda g2_coord_y
    cmp #CO_GOA_MAX_Y
    bcs labe2
    inc g2_coord_y
    rts
lac09:
lac0a = * + 1
lac0b = * + 2
!by $00, $00, $00
cpu_skill_hmm_i:
!by $00
cpu_shoot_delay:
!by $00
lac0e:
!by $10, $12, $15, $17, $1a, $1c, $1e, $21, $23, $26, $28, $2a, $2d, $2f, $32, $34
lac1e:
!by $20, $1e, $1c, $1a, $18, $16, $14, $12, $10, $0e, $0c, $0a, $08, $06, $04, $02
lac2e:
!by $20, $1e, $1c, $1a, $18, $16, $14, $12, $10, $0e, $0c, $0a, $08, $06, $04, $02
tbl_cpu_shoot_delay:
!by $01, $01, $01, $01, $01, $01, $01, $01, $01, $01, $01, $0a, $0a, $0a, $0a, $0a

cpu_check_shoot_puck:
    lda flag_cpu_player
    clc
    adc #$01
    eor #$03
    cmp pX_has_puck
    bne cpu_dont_shoot_puck
    lda cpu_shoot_delay
    beq lac68
    dec cpu_shoot_delay
    bne cpu_dont_shoot_puck
    lda #$10  ; fire
    rts
lac68:
    jsr cpu_shoot_check_coord_for_delay
    bcc cpu_dont_shoot_puck
    ldx cpu_skill_hmm_i
    lda tbl_cpu_shoot_delay,x
    sta cpu_shoot_delay
cpu_dont_shoot_puck:
    lda #$00  ; do not fire
    rts

cpu_shoot_check_coord_for_delay:
    ldx p1_coord_x
    ldy p1_coord_y
    lda flag_cpu_player
    bne lac8a
    ldx p2_coord_x
    ldy p2_coord_y
lac8a:
    cpy #CO_CPU_SHOOT_MIN_Y
    bcc cpu_shoot_keep_delay_0
    cpy #CO_CPU_SHOOT_MAX_Y
    bcs cpu_shoot_keep_delay_0
    lda flag_cpu_player
    bne lac9d
    cpx #CO_CPU_SHOOT_MIN_X
    bcs cpu_shoot_keep_delay_0
cpu_shoot_add_delay:
    sec
    rts
lac9d:
    cpx #CO_CPU_SHOOT_MAX_X
    bcs cpu_shoot_add_delay
cpu_shoot_keep_delay_0:
    clc
    rts

tbl_cpu_skill_hmm1:
!by $00, $07, $0d
tbl_cpu_skill_hmm2:
!by $0a, $0c, $0f

adjust_cpu_skill_hmm:
    lda flag_num_players
    bne lacd3
    lda phase_hmm
    bne lacd3
    lda timer_sub_60
    bne lacd3
    lda timer_s
    beq lacc5
    cmp #$20
    beq lacc5
    cmp #$40
    bne lacd3
lacc5:
    ldy flag_skill
    lda cpu_skill_hmm_i
    cmp tbl_cpu_skill_hmm2,y
    bcs lacd3
    inc cpu_skill_hmm_i
lacd3:
    rts

init_cpu_hmm:
    lda flag_num_players
    bne lad07
    ldx score_bcd_p1
    ldy score_bcd_p2
    lda flag_cpu_player
    beq lacea
    ldx score_bcd_p2
    ldy score_bcd_p1
lacea:
    stx temp_x
    cpy temp_x
    beq lad07
    bcs lacfe
    ldy flag_skill
    lda tbl_cpu_skill_hmm2,y
    sta cpu_skill_hmm_i
    rts
lacfe:
    ldy flag_skill
    lda tbl_cpu_skill_hmm1,y
    sta cpu_skill_hmm_i
lad07:
    rts

init_vars:
    lda #CO_PL1_START_X
    sta p1_coord_x
    lda #CO_PL2_START_X
    sta p2_coord_x
    lda #CO_PLG_START_Y
    sta p1_coord_y
    sta p2_coord_y
    sta g1_coord_y
    sta g2_coord_y
    lda #$02
    sta p1_dir_hmm
    sta p1_dir_i
    lda #$06
    sta p2_dir_hmm
    sta p2_dir_i
    lda #$00
    sta p1_acc_x
    sta p1_acc_y
    sta p1_vel_x
    sta p1_vel_y
    sta p2_acc_x
    sta p2_acc_y
    sta p2_vel_x
    sta p2_vel_y
    sta timeout_check_collider_puck_p1
    sta timeout_check_collider_puck_p2
    sta timer_sub_60
    sta counter_hmm_sub
    sta puck_holder
    sta timeout_check_coll_p1_p2
    sta lac0b
    sta lac09
    sta lac0a
    sta cpu_shoot_delay
    jsr draw_saves
    lda #$04
    sta counter_hmm
    jsr set_p1_sprptr
    jsr set_p2_sprptr
    lda #$00
    sta phase_hmm
    lda #$00
    sta l044a
    jmp start_puck

entry2:
    lda #$00
    sta flag_num_players
    sta flag_skill
    lda #$01
    sta flag_music
    lda #SOUND_VOLUME
    sta $900e
entry3:
    lda #$00
    sta score_bcd_p1
    sta score_bcd_p2
    sta saves_p1
    sta saves_p2
    sta flag_overtime
    sta flag_skip_irq_logic
    ldy flag_skill
    lda tbl_cpu_skill_hmm1,y
    sta cpu_skill_hmm_i
    lda #$00
    jsr music_init
    jsr hook_irq_handler_nop
    lda #$00
    sta flag_skip_irq_logic
    sta flag_skip_irq_logic2
    cli
    jsr wait_for_irq_x2
    ldx #$00
    jsr setup_screen_x
    jsr hook_irq_handler_ingame
    jsr main_menu
    jsr music_stop
    jsr timer_init_3_00_draw
    lda #$01
    jsr music_init
    jsr hook_irq_handler_nop
    cli
    jsr wait_for_irq_x2
    ldx #$01
    jsr setup_screen_x
    jmp init_game_loop

init_music_and_game_loop:
    lda #$01
    jsr music_init
    jsr hook_irq_handler_nop
    cli
    jsr wait_for_irq_x2
;    jsr screen_disable

init_game_loop:
    jsr init_sprite_regs
    jsr init_vars
    jsr init_cpu_hmm
    jsr fixup_px_sprs
    jsr hook_irq_handler_ingame
    jsr screen_enable
game_loop:
    inc flag_wait_for_irq
-   lda flag_wait_for_irq
    bne -
    jsr draw_sprites
!if DEBUG_IRQ_LEN > 0 {
    lda #$1f
    sta $900f
}
;    jsr delay
    jsr check_collider_foo3
    jsr check_shoot_puck
    jsr move_free_puck
    jsr check_puck_limits
    jsr handle_puck
    jsr move_free_puck
    jsr check_puck_limits
    jsr handle_puck
    jsr get_input_p1_cond
    jsr get_input_p2_cond
    jsr cond_check_coll_p1_p2
    jsr check_coll_wall_p1
    jsr check_coll_wall_p2
    jsr move_p1_cond
    jsr move_p2_cond
    jsr move_goalies
    jsr check_pause_key
    jsr l15ee
    jsr set_p1_sprptr
    jsr set_p2_sprptr
    jsr draw_scores
    jsr timer_tick
    jsr draw_timer
    jsr adjust_cpu_skill_hmm
    jsr fixup_px_sprs
!if DEBUG_IRQ_LEN > 0 {
    lda #$1b
    sta $900f
}
    jmp game_loop


fixup_px_sprs:
    lda p2_anim_cooked
    lsr
    lsr
    lsr
    and #7
    tax
    lda tbl_spryoff,x
    clc
    adc p2_coord_y
    sta p2_coord_y_cooked

    lda tbl_sprh,x
    sta p2_spr_h

    lda p1_anim_cooked
    lsr
    lsr
    lsr
    and #7
    tax
    lda tbl_spryoff,x
    clc
    adc p1_coord_y
    sta p1_coord_y_cooked

    lda tbl_sprh,x
    sta p1_spr_h
    sta draw_gfx_smc_h3
    sta draw_gfx_smc_h4
    sta draw_gfx_smc_h6
    sta draw_gfx_smc_h8
    sec
    sbc #8
    sta draw_gfx_smc_h1
    sta draw_gfx_smc_h2
    sta draw_gfx_smc_h5
    sta draw_gfx_smc_h7

    rts


; x/y = coords
-   rts
draw_puck:
    cpy #CO_PUCK_MIN_Y
    bcc -
    cpy #CO_PUCK_MAX_Y + 1
    bcs -
    txa
    lsr
    lsr
    lsr
    cmp #AREA_W - 1
    bcs -
    sta temp_x
    txa
    and #7
    sta temp_xs
    tax
    lda sprite_puck,x
    sta temp_ch

    tya
    lsr
    lsr
    lsr
    tax
    lda tbl_y_char_l,x
    clc
    adc temp_x
    sta zpdst
    lda tbl_y_char_h,x
    adc #0
    sta zpdsth
    ldx temp_x

    tya
    and #7
    sta temp_ys
    cmp #(8 - Sprite_h_puck + 1)
    bcc +

    cpx #0
    beq ++

    ldy #COLS
    lda #(CHAR_BASE_GFX + 1)
    sta (zpdst),y

    cpx #AREA_W - 2
    bcs +++
++
    lda temp_xs
    cmp #7
    bne +++

    ldy #COLS + 1
    lda #(CHAR_BASE_GFX + 3)
    sta (zpdst),y
    bne ++  ; bra
+
    lda temp_xs
    cmp #7
    bne +++
++
    cpx #AREA_W - 2
    bcs +++
    ldy #1
    lda #(CHAR_BASE_GFX + 2)
    sta (zpdst),y
+++
    cpx #0
    beq +
    ldy #0
    lda #(CHAR_BASE_GFX + 0)
    sta (zpdst),y

+
    ldx #0
    ldy temp_ys
    beq +
    lda #0
-   sta charset + CHAR_BASE_GFX * 8,x
    inx
    dey
    bne -
+   lda temp_ch
!for .n, Sprite_h_puck {
    sta charset + CHAR_BASE_GFX * 8,x
    inx
}
    lda #0
-   sta charset + CHAR_BASE_GFX * 8,x
    inx
    cpx #16
    bne -

    ldy temp_ys
    beq +
    lda #0
-   sta charset + CHAR_BASE_GFX * 8,x
    inx
    dey
    bne -
+   lda #%10000000
!for .n, Sprite_h_puck {
    sta charset + CHAR_BASE_GFX * 8,x
    inx
}
    lda #0
-   sta charset + CHAR_BASE_GFX * 8,x
    inx
    cpx #32
    bne -

    rts

; y = y coord
; x = x char coord
; a = goalie l/r 0/1
draw_goalie:
    stx temp_x
    cmp #0
    beq +
    lda #<sprite_goalie_r
    ldx #>sprite_goalie_r
    bne ++
+   lda #<sprite_goalie_l
    ldx #>sprite_goalie_l
++  sta zpsrc
    stx zpsrch
    ; C64 goalie sprite has 7 empty lines on top
    tya
    clc
    adc #7
    tay
    lda #1
    jmp draw_gfx_start

; x/y = coords
; a = (dir << 5) | (frame << 3)
; temp_ch = first available char
draw_player:
    stx temp_x
    sta temp_f
    txa
    and #7
    ora temp_f
    tax
draw_player_smc_tbl_ll = * + 1
draw_player_smc_tbl_lh = * + 2
    lda tbl_spraddr_l,x
    sta zpsrc
draw_player_smc_tbl_h = * + 2
    lda tbl_spraddr_h,x
    sta zpsrch
    lsr temp_x
    lsr temp_x
    lsr temp_x
    lda #2
    bcc draw_gfx_start
    lda #3

; draw_gfx_smc_h1 = draw_gfx_smc_h2 = gfx h - 8
; a = x width in chars
; y = y coord
; temp_x = x char coord
; zpsrc/h = pointer to gfx
draw_gfx_start:
    sta temp_cw
    tya
    and #7
    sta temp_ys
    tya
    lsr
    lsr
    lsr
    tax
    lda tbl_y_char_l,x
    clc
    adc temp_x
    sta zpdst
    lda tbl_y_char_h,x
    adc #0
    sta zpdsth

draw_gfx_loop_x:
    ldy #0
    sty draw_gfx_smc_dptr_y
    sty draw_gfx_smc_sptr_y
    lda (zpdst),y
    bne draw_gfx_start_over
    lda temp_ch
    sta (zpdst),y
    inc temp_ch
    sec
    sbc #CHAR_BASE_GFX
    ; NOTE:
    ; If y gets too large, the read char may be < CHAR_BASE_GFX.
    ; "bcc skip_this_one" is avoided in interest of speed
    ; and code is moved away from the disaster area (up to $1de7?).
    asl
    asl
    asl
    tax
    bcs draw_gfx_start_offs100

    ldy temp_ys
    bne +
    jmp draw_gfx_skip_start_partial
+   lda #0
-   sta charset + CHAR_BASE_GFX * 8,x
    inx
    dey
    bne -
    lda #8
    sec
    sbc temp_ys
    sta temp_cl
    ldy #0
-   lda (zpsrc),y
    sta charset + CHAR_BASE_GFX * 8,x
    inx
    iny
    dec temp_cl
    bne -
    jmp draw_gfx_loop_y

draw_gfx_start_offs100:
    ldy temp_ys
    bne +
    jmp draw_gfx_skip_start_partial_offs100
+   lda #0
-   sta charset + CHAR_BASE_GFX * 8 + $100,x
    inx
    dey
    bne -
    lda #8
    sec
    sbc temp_ys
    sta temp_cl
    ldy #0
-   lda (zpsrc),y
    sta charset + CHAR_BASE_GFX * 8 + $100,x
    inx
    iny
    dec temp_cl
    bne -
    jmp draw_gfx_loop_y

draw_gfx_start_over:
    sec
    sbc #CHAR_BASE_GFX
    asl
    asl
    asl
    tax
    bcs draw_gfx_start_over_offs100

    ldy temp_ys
    beq draw_gfx_skip_start_partial_or
    lda #8
    sec
    sbc temp_ys
    sta temp_cl
    txa
    clc
    adc temp_ys
    tax
    ldy #0
-   lda (zpsrc),y
    ora charset + CHAR_BASE_GFX * 8,x
    sta charset + CHAR_BASE_GFX * 8,x
    inx
    iny
    dec temp_cl
    bne -
    jmp draw_gfx_loop_y

draw_gfx_start_over_offs100:
    ldy temp_ys
    bne +
    jmp draw_gfx_skip_start_partial_or_offs100
+   lda #8
    sec
    sbc temp_ys
    sta temp_cl
    txa
    clc
    adc temp_ys
    tax
    ldy #0
-   lda (zpsrc),y
    ora charset + CHAR_BASE_GFX * 8 + $100,x
    sta charset + CHAR_BASE_GFX * 8 + $100,x
    inx
    iny
    dec temp_cl
    bne -
    jmp draw_gfx_loop_y

draw_gfx_over:
    sec
    sbc #CHAR_BASE_GFX
    asl
    asl
    asl
    tax

draw_gfx_smc_sptr_y = * + 1
    ldy #0
    bcs draw_gfx_over_offs100
draw_gfx_smc_h1 = * + 1
    cpy #(Sprite_h_player - 8)
    bcs +
draw_gfx_skip_start_partial_or:
!for .n, 8 {
    lda (zpsrc),y
    ora charset + CHAR_BASE_GFX * 8 + .n - 1,x
    sta charset + CHAR_BASE_GFX * 8 + .n - 1,x
    iny
}
    jmp draw_gfx_loop_y
+

    ; draw last partial char with or
-   lda (zpsrc),y
    ora charset + CHAR_BASE_GFX * 8,x
    sta charset + CHAR_BASE_GFX * 8,x
    inx
    iny
draw_gfx_smc_h4 = * + 1
    cpy #Sprite_h_player
    bne -
    jmp draw_gfx_next_x_char

draw_gfx_over_offs100:
draw_gfx_smc_h7 = * + 1
    cpy #(Sprite_h_player - 8)
    bcs +
draw_gfx_skip_start_partial_or_offs100:
!for .n, 8 {
    lda (zpsrc),y
    ora charset + CHAR_BASE_GFX * 8 + .n - 1 + $100,x
    sta charset + CHAR_BASE_GFX * 8 + .n - 1 + $100,x
    iny
}
    jmp draw_gfx_loop_y
+

    ; draw last partial char with or
-   lda (zpsrc),y
    ora charset + CHAR_BASE_GFX * 8 + $100,x
    sta charset + CHAR_BASE_GFX * 8 + $100,x
    inx
    iny
draw_gfx_smc_h8 = * + 1
    cpy #Sprite_h_player
    bne -
    jmp draw_gfx_next_x_char


draw_gfx_loop_y:
    sty draw_gfx_smc_sptr_y
    lda draw_gfx_smc_dptr_y
    clc
    adc #COLS
    sta draw_gfx_smc_dptr_y
draw_gfx_smc_dptr_y = * + 1
    ldy #0
    lda (zpdst),y
    beq +
    jmp draw_gfx_over
+   lda temp_ch
    sta (zpdst),y
    inc temp_ch
    sec
    sbc #CHAR_BASE_GFX
    asl
    asl
    asl
    tax
    bcs draw_gfx_offs100

    ldy draw_gfx_smc_sptr_y
draw_gfx_smc_h2 = * + 1
    cpy #(Sprite_h_player - 8)
    bcs draw_gfx_last
draw_gfx_skip_start_partial:
!for .n, 8 {
    lda (zpsrc),y
    sta charset + CHAR_BASE_GFX * 8 + .n - 1,x
    iny
}
    jmp draw_gfx_loop_y

draw_gfx_offs100:
    ldy draw_gfx_smc_sptr_y
draw_gfx_smc_h5 = * + 1
    cpy #(Sprite_h_player - 8)
    bcs draw_gfx_last_offs100
draw_gfx_skip_start_partial_offs100:
!for .n, 8 {
    lda (zpsrc),y
    sta charset + CHAR_BASE_GFX * 8 + .n - 1 + $100,x
    iny
}
    jmp draw_gfx_loop_y

draw_gfx_last:
    ; draw last partial char
-   lda (zpsrc),y
    sta charset + CHAR_BASE_GFX * 8,x
    inx
    iny
draw_gfx_smc_h3 = * + 1
    cpy #Sprite_h_player
    bne -
    txa
    and #7
    beq draw_gfx_next_x_char
    ; clear rest of char
    tay
    lda #0
-   sta charset + CHAR_BASE_GFX * 8,x
    inx
    iny
    cpy #8
    bne -
    beq draw_gfx_next_x_char

draw_gfx_last_offs100:
    ; draw last partial char
-   lda (zpsrc),y
    sta charset + CHAR_BASE_GFX * 8 + $100,x
    inx
    iny
draw_gfx_smc_h6 = * + 1
    cpy #Sprite_h_player
    bne -
    txa
    and #7
    beq draw_gfx_next_x_char
    ; clear rest of char
    tay
    lda #0
-   sta charset + CHAR_BASE_GFX * 8 + $100,x
    inx
    iny
    cpy #8
    bne -
    beq draw_gfx_next_x_char

draw_gfx_next_x_char:
    dec temp_cw
    beq ++
    ; next sprite line
    lda zpsrc
    clc
    adc px_spr_h ;#Sprite_h_player
    sta zpsrc
    bcc +
    inc zpsrch
+
    ; next vmatrix column
    inc zpdst
    bne +
    inc zpdsth
+   jmp draw_gfx_loop_x
++
    rts

draw_sprites:
!if DEBUG_IRQ_LEN > 0 {
    lda #$1d
    sta $900f
}
    lda #CHAR_EMPTY
    jsr draw_clear
!if DEBUG_IRQ_LEN > 0 {
    lda #$1c
    sta $900f
}
    ldx puck_coord_x
    ldy puck_coord_y
    jsr draw_puck

    lda #(CHAR_BASE_GFX + 4)
    sta temp_ch

    lda p1_spr_h
    sta px_spr_h
    lda p1_anim_cooked
    ldx p1_coord_x
    ldy p1_coord_y_cooked
    jsr draw_player

draw_player_2_tbl_hll = * + 1
    ldy #<tbl_spraddr_l
    sty draw_player_smc_tbl_ll
draw_player_2_tbl_hlh = * + 1
    ldy #>tbl_spraddr_l
    sty draw_player_smc_tbl_lh
draw_player_2_tbl_hh = * + 1
    ldy #>tbl_spraddr_h
    sty draw_player_smc_tbl_h

    lda p2_spr_h
    sta px_spr_h
    sta draw_gfx_smc_h3
    sta draw_gfx_smc_h4
    sta draw_gfx_smc_h6
    sta draw_gfx_smc_h8
    sec
    sbc #8
    sta draw_gfx_smc_h1
    sta draw_gfx_smc_h2
    sta draw_gfx_smc_h5
    sta draw_gfx_smc_h7

    lda p2_anim_cooked
    ldx p2_coord_x
    ldy p2_coord_y_cooked
    jsr draw_player

    lda #(Sprite_h_goalie - 8)
    sta draw_gfx_smc_h1
    sta draw_gfx_smc_h2
    sta draw_gfx_smc_h5
    sta draw_gfx_smc_h7
    lda #Sprite_h_goalie
    sta draw_gfx_smc_h3
    sta draw_gfx_smc_h4
    sta draw_gfx_smc_h6
    sta draw_gfx_smc_h8

    lda #0
    ldx #1
    ldy g1_coord_y
    jsr draw_goalie

    lda #1
    ldx #COLS - 2
    ldy g2_coord_y
    jsr draw_goalie

!if DEBUG_IRQ_LEN > 0 {
    lda #$1b
    sta $900f
}

    ldy #<tbl_spraddr_l
    sty draw_player_smc_tbl_ll
    ldy #>tbl_spraddr_l
    sty draw_player_smc_tbl_lh
    ldy #>tbl_spraddr_h
    sty draw_player_smc_tbl_h

    rts

; ###############

!src "sound.asm"
!src "music.asm"

; ###############

; # data

!align $1f, 0

!src "gfx_goalie.asm"

; 1 byte of unique data + xshifted versions
sprite_puck:
!by %##......
!by %.##.....
!by %..##....
!by %...##...
!by %....##..
!by %.....##.
!by %......##
!by %.......#

Main_high_end = *

; ###############

draw_clear:
; speedcode is generated here

; ###############

Sprsrc_player:
!src "gfx_player.asm"

Sprsrc_player_end = *

!zn Phase1_Init {   ; all within the zone can be overwritten after run once

; # p1 code
Init_phase1:

!pseudopc vmatrix {
    ; ### build sprite gfx
    ; NOTE: code is for player 2 gfx (with overlay), this is done first and the code patched before the second (p1 gfx) call.

    lda #0
    sta temp_spri
    sta temp_ndir
init_sprgfx_dir_loop:
    lda #3
    sta temp_nframe
init_sprgfx_frame_loop:

    ; copy sprite data, convert:
    ;  0  1      0 21
    ;  2  3  ->  1 22
    ;       ...
    ; 40 41     20 42
    ldx #0
init_sprgfx_copy_loop:
    ldy #0
    lda (zpsrc),y
    inc zpsrc
    bne +
    inc zpsrch
+
    sta+2 temp_gfx_c0,x
    lda (zpsrc),y
    inc zpsrc
    bne +
    inc zpsrch
+
    sta+2 temp_gfx_c1,x
    lda #0
    sta+2 temp_gfx_c2,x
    inx
    cpx #Sprite_h_player
    bne init_sprgfx_copy_loop

init_sprgfx_eor_op:
    jsr init_sprgfx2_eor

    ldx temp_ndir
    lda tbl_spryoff,x
    sta temp_spryoff
    lda tbl_sprh,x
    sta temp_sprh
    clc
    adc temp_spryoff
    sta temp_sprylast

    ; make xshifted copies
    ldy #0
    beq init_sprgfx_copy_shifted    ; skip first shift

init_sprgfx_xshift_loop:
    ldx #(Sprite_h_player - 1)
-   lsr temp_gfx_c0,x
    ror temp_gfx_c1,x
    ror temp_gfx_c2,x
    dex
    bpl -

init_sprgfx_copy_shifted:
init_sprgfx_copy_addr_ptr = * + 1
init_sprgfx_copy_addr_ptrh = * + 2
    jsr init_sprgfx2_copy_addr
    sty temp_nxshift
    lda #temp_gfx_c0
    jsr init_sprgfx_copy_column
    lda #temp_gfx_c1
    jsr init_sprgfx_copy_column
    ldy temp_nxshift
    cpy #4
    bcc +
    lda #temp_gfx_c2
    jsr init_sprgfx_copy_column
    ldy temp_nxshift
+
    iny
    cpy #8
    bne init_sprgfx_xshift_loop

    dec temp_nframe
    beq +
    jmp init_sprgfx_frame_loop
+

    ; next overlay
    lda temp_tptr
    clc
    adc #Sprite_h_player * 2
    sta temp_tptr
    bcc +
    inc temp_tptrh
+

    ldx temp_ndir
    inx
    stx temp_ndir
    cpx #8
    beq +
    jmp init_sprgfx_dir_loop
+
    rts ; jmp init_phase2

; # p1 subroutines

init_sprgfx_copy_column:
    sta init_sprgfx_copy_column_smc
    ldy #0
    ldx temp_spryoff
init_sprgfx_copy_column_smc = * + 1
-   lda+2 temp_gfx_c0,x
    sta (zpdst),y
    inx
    iny
    cpy temp_sprh
    bne -
    tya
    jsr init_sprgfx_add_dptr
    rts

init_sprgfx_copy_addr:
    ldx temp_spri
    lda temp_spri_tbl,x
    tax
    lda zpdst
    sta tbl_spraddr_l,x
    lda zpdsth
    sta tbl_spraddr_h,x
    inc temp_spri
    rts

init_sprgfx_add_dptr:
    clc
    adc zpdst
    bcc +
    inc zpdsth
+   sta zpdst
    rts

} ; !pseudopc

IP1_End = *
IP1_LEN = IP1_End - Init_phase1

!if IP1_LEN > $200 {
!error "too long ip1"
!error IP1_LEN - $200
}

} ; !zn Phase1_Init

!zn Phase0_Init {   ; all within the zone can be overwritten after run once

; # p0 code
init_phase0:
    sei
    ldx #$ff
    txs
    cld
    lda #$7f
    sta $913d
    sta $913e

    ; initialize VIC/VIA
    ldx #$f
-   lda gamefw_tv_tbl_vic,x
    sta $9000,x
    dex
    bpl -
    lda #0
    ldx #$a
-   sta $9132,x
    dex
    bpl -

    lda #0
    sta $9122
;    ldx #$ff
    stx $9121
    stx $9123

    ; clear colorram
;    lda #0
    tax
-   sta colorram,x
    sta colorram + $100,x
    inx
    bne -

    ; setup running counter on VIA 2
    lda #$40
;    sta $911b       ; enable Timer A free run on VIA 2
    lda #$ff
    sta $9116
    sta $9115

    ; setup int vectors
    lda #<irq_handler_ingame
    sta $0314
    lda #>irq_handler_ingame
    sta $0315
    lda #<brk_handler
    sta $0316
    lda #>brk_handler
    sta $0317

    ; copy phase 1 code to vmatrix
    ldx #0
-   lda Init_phase1,x
    sta vmatrix,x
!if IP1_LEN > $100 {
    lda Init_phase1 + $100,x
    sta vmatrix + $100,x
}
    inx
    bne -

    ; build sprite yoffset and height luts
    ; NOTE: assumes first frame is as tall/up as all
build_spr_luts:
    lda #<Sprsrc_player
    sta zpdst
    lda #>Sprsrc_player
    sta zpdsth

    lda #0
    sta temp_spri
--  ldx #0
    ldy #0
-   lda (zpdst),y
    iny
    ora (zpdst),y
    bne +
    inx
    iny
    bne -   ; bra
+
    txa
    ldx temp_spri
    sta tbl_spryoff,x

    ldx #Sprite_h_player
    ldy #Sprite_h_player * 2 - 1
-   lda (zpdst),y
    dey
    ora (zpdst),y
    bne +
    dex
    dey
    bne -   ; bra
+

    txa
    ldx temp_spri
    sec
    sbc tbl_spryoff,x
    sta tbl_sprh,x
    cpx #7
    beq +
    inc temp_spri

    lda #Sprite_h_player * 2 * 3    ; skip other frames
    jsr init_sprgfx_add_dptr
    jmp --
+

    ; build sprite index lut
    lda #0
    sta temp_ndir
    tay
---
    ldx #0
    stx temp_nframe
--  lda p0_spri_f,x
    ldx temp_ndir
    ora p0_spri_d,x
    sta temp_tptr
    ldx #0
-   txa
    ora temp_tptr
    sta temp_spri_tbl,y
    iny
    inx
    cpx #8
    bne -
    ldx temp_nframe
    inx
    stx temp_nframe
    cpx #3
    bne --
    ldx temp_ndir
    inx
    stx temp_ndir
    cpx #8
    bne ---


    ; test if 24k is available
    ldx #$55
    stx $6000
    cpx $6000
    bne init_phase0_noextgfx
    inx
    stx $6000
    cpx $6000
    beq init_phase0_gotext
init_phase0_noextgfx:
    jmp init_phase0_prepare_p1

init_phase0_gotext:
    ; ### build sprite gfx for player 2

    lda #<sprsrc_player2_over
    sta temp_tptr
    lda #>sprsrc_player2_over
    sta temp_tptrh

    lda #<Sprsrc_player
    sta zpsrc
    lda #>Sprsrc_player
    sta zpsrch
    lda #<Spritegfx2
    sta zpdst
    lda #>Spritegfx2
    sta zpdsth

    jsr vmatrix ; use p1 gfx code, already prepared otherwise

    ; point drawing code to player 2 gfx
    lda #<tbl_spraddr2_l
    sta draw_player_2_tbl_hll
    lda #>tbl_spraddr2_l
    sta draw_player_2_tbl_hlh
    lda #>tbl_spraddr2_h
    sta draw_player_2_tbl_hh

init_phase0_prepare_p1:
    ; prepare phase 1

    lda #<Sprsrc_player
    sta zpsrc
    lda #>Sprsrc_player
    sta zpsrch
    lda #<Spritegfx
    sta zpdst
    lda #>Spritegfx
    sta zpdsth

    ; modify code for player 1 gfx conversion
    lda #OP_BIT_ABS
    sta init_sprgfx_eor_op
    lda #<init_sprgfx_copy_addr
    sta init_sprgfx_copy_addr_ptr
    lda #>init_sprgfx_copy_addr
    sta init_sprgfx_copy_addr_ptrh

    ; set return address to phase2
    lda #>(init_phase2 - 1)
    pha
    lda #<(init_phase2 - 1)
    pha

    jmp vmatrix ; relocated Init_phase1


; # p0 subroutines

; these are called by init_sprgfx at vmatrix when(/if) generating p2 gfx

init_sprgfx2_eor:
    ; overlay sprite data, convert:
    ;  0  1      0 21
    ;  2  3  ->  1 22
    ;       ...
    ; 40 41     20 42
    ldx #0
    ldy #0
init_sprgfx2_eor_loop:
    lda temp_gfx_c0,x
    ora (temp_tptr),y
    sta temp_gfx_c0,x
    iny
    lda temp_gfx_c1,x
    ora (temp_tptr),y
    sta temp_gfx_c1,x
    iny
    inx
    cpx #Sprite_h_player
    bne init_sprgfx2_eor_loop
    rts

init_sprgfx2_copy_addr:
    ldx temp_spri
    lda temp_spri_tbl,x
    tax
    lda zpdst
    sta tbl_spraddr2_l,x
    lda zpdsth
    sta tbl_spraddr2_h,x
    inc temp_spri
    rts

; # p0 data

gamefw_tv_tbl_vic:
!by 12, 37, 0, 21 << 1, 0
!by ((((vmatrix & $1c00) >> 10) | ((vmatrix & $8000) >> 12) XOR 8) << 4) | (((charset & $1c00) >> 10) | ((charset & $8000) >> 12) XOR 8)
!by 0, 0, 0, 0
!by 0, 0, 0, 0
!by 0, $1b

p0_spri_f:
!by (0 << 6), (1 << 6), (2 << 6)

p0_spri_d:
!by (0 << 3), (1 << 3), (2 << 3), (3 << 3), (4 << 3), (5 << 3), (6 << 3), (7 << 3)

sprsrc_player2_over:
!src "gfx_plover.asm"

IP0_End = *
} ; !zn Phase0_Init


!if * > RAM_16k_end {
!error "ip0 over RAM end"
!error * - RAM_16k_end
}
